@@ -70,7 +70,7 @@ def _find_max_delay(delays):
 	if all(symengine.sympify(delay).is_Number for delay in delays):
 		return float(symengine.sympify(max(delays)).n(real=True))
 	else:
-		raise ValueError("Delay depends on time or dynamics; cannot determine max_delay automatically. You have to pass it as an argument to jitcdde.")
+		raise ValueError("Delay depends on time, dynamics, or control parameters; cannot determine max_delay automatically. You have to pass it as an argument to jitcdde.")
 
 def _propagate_delays(delays, p, threshold=1e-5):
 	result = [0]
@@ -166,7 +166,7 @@ class jitcdde(jitcxde):
 		Whether JiTCDDE shall give progress reports on the processing steps.
 	
 	module_location : string
-		location of a module file from which functions are to be loaded (see `save_compiled`). If you use this, you need not give `f_sym` as an argument, but then you must give `n` and `max_delay`. Also note that the integrator may lack some functionalities, depending on the arguments you provide.
+		location of a module file from which functions are to be loaded (see `save_compiled`). If you use this, you need not give `f_sym` as an argument, but then you must give `n` and `max_delay`. If you used `control_pars` or `callback_functions`, you have to provide them again. Also note that the integrator may lack some functionalities, depending on the arguments you provide.
 	"""
 	
 	dynvar = current_y
@@ -215,7 +215,8 @@ class jitcdde(jitcxde):
 	def delays(self, new_delays):
 		self._delays = new_delays
 		if (self._delays is not None) and (0 not in self._delays):
-			self._delays.append(0)
+			self._delays = np.append(self._delays,0)
+		self._max_delay = None
 	
 	@property
 	def max_delay(self):
@@ -313,7 +314,7 @@ class jitcdde(jitcxde):
 		----------
 		function : callable or iterable of symbolic expressions
 			If callable, this takes the time as an argument and returns an iterable of floats that is the initial state of the past at that time.
-			If an iterable of expressions, each expression represents how initial past of the respective component depends on `t` (requires SymPy).
+			If an iterable of expressions, each expression represents how the initial past of the respective component depends on `t` (requires SymPy).
 			In both cases, the lengths of the iterable must match the dimension of the differential equation (`n`).
 			
 		times_of_interest : iterable of numbers or `None`
@@ -340,7 +341,7 @@ class jitcdde(jitcxde):
 		"""
 		Returns an object that represents all anchors currently used by the integrator, which compeletely define the current state. The object is a `CubicHermiteSpline <https://chspy.readthedocs.io>`_ instance (with a few special extensions for JiTCDDE), which allows you to extract all sorts of information from it if you want.
 		
-		The format can also be used as an argument for `add_past_points`. An example where this is useful is when you want to switch between plain integration and one that also obtains Lyapunov exponents.
+		The format can also be used as an argument for `add_past_points`. An example where this is useful is when you want to switch between plain integration and one that also obtains Lyapunov exponents. You can also use this to implement time-dependent equations, however, you need to be careful to truncate the result properly. Moreover, if your delay changes, you may need to set the `max_delay` accordingly to avoid too much past being discarded before you call this method.
 		
 		If you reinitialise this integrator after calling this, this past will be used.
 		"""
@@ -791,7 +792,7 @@ class jitcdde(jitcxde):
 			warn("The target time is smaller than the current time. No integration step will happen. The returned state will be extrapolated from the interpolating Hermite polynomial for the last integration step. You may see this because you try to integrate backwards in time, in which case you did something wrong. You may see this just because your sampling step is small, in which case there is no need to worry.")
 		
 		if not self.initial_discontinuities_handled:
-			warn("You did not explicitly handle initial discontinuities. Proceed only if you know what you are doing. This is only fine if you somehow chose your initial past such that the derivative of the last anchor complies with the DDE. In this case, you can set the attribute `initial_discontinuities_handled` to `False` to suppress this warning. See https://jitcdde.rtfd.io/#discontinuities for details.")
+			warn("You did not explicitly handle initial discontinuities. Proceed only if you know what you are doing. This is only fine if you somehow chose your initial past such that the derivative of the last anchor complies with the DDE. In this case, you can set the attribute `initial_discontinuities_handled` to `True` to suppress this warning. See https://jitcdde.rtfd.io/#discontinuities for details.")
 		
 		while self.DDE.get_t() < target_time:
 			if self.try_single_step(self.dt):
@@ -937,7 +938,7 @@ class jitcdde(jitcxde):
 			warn("The max_step parameter is retired and does not need to be used anymore. Instead, step_on_discontinuities now adapts the step size. This will raise an error in the future.")
 		
 		if not all(symengine.sympify(delay).is_number for delay in self.delays):
-			raise ValueError("At least one delay depends on time or dynamics; cannot automatically determine steps.")
+			raise ValueError("At least one delay depends on time, dynamics, or control parameters; cannot automatically determine steps.")
 		self.delays = [
 				# This conversion is due to SymEngine.py issue #227
 				float(symengine.sympify(delay).n(real=True))
@@ -1523,34 +1524,42 @@ class jitcdde_input(jitcdde):
 			kwargs["max_delay"] = max( kwargs["max_delay"], self.input_duration )
 		
 		super().__init__( f_full, n=self.n, **kwargs )
+		
+		self._past = None
 	
 	def initiate_past(self):
 		pass
 	
 	@property
 	def past(self):
-		assert len(self.regular_past)>1, "You need to add at least two past points first. Usually this means that you did not set an initial past at all."
-		if self.regular_past[-1].time != 0:
-			raise ValueError("For jitcdde_input, the initial past must end at t=0.")
-		return chspy.join(self.regular_past,self.input_past)
+		if self._past is None:
+			assert len(self.regular_past)>1, "You need to add at least two past points first. Usually this means that you did not set an initial past at all."
+			if self.regular_past[-1].time != 0:
+				raise ValueError("For jitcdde_input, the initial past must end at t=0.")
+			self._past = chspy.join(self.regular_past,self.input_past)
+		return self._past
 	
 	@past.setter
 	def past(self,value):
 		raise AssertionError("For jitcdde_input, the past attribute should not be directly set.")
 	
 	def add_past_point(self, time, state, derivative):
+		self._past = None
 		self.reset_integrator(idc_unhandled=True)
 		self.regular_past.add((time,state,derivative))
 
 	def add_past_points(self, anchors):
+		self._past = None
 		self.reset_integrator(idc_unhandled=True)
 		for anchor in anchors:
 			self.regular_past.add(anchor)
 	
 	def constant_past(self,state,time=0):
+		self._past = None
 		self.regular_past.constant(state,time)
 	
 	def past_from_function(self,function,times_of_interest=None,max_anchors=100,tol=5):
+		self._past = None
 		if times_of_interest is None:
 			times_of_interest = np.linspace(-self.max_delay,0,10)
 		else:
@@ -1561,6 +1570,7 @@ class jitcdde_input(jitcdde):
 	def get_state(self):
 		self._initiate()
 		self.DDE.forget(self.max_delay)
+		self._past = None
 		self.regular_past.clear()
 		for anchor in self.DDE.get_full_state():
 			self.regular_past.append((
@@ -1571,6 +1581,7 @@ class jitcdde_input(jitcdde):
 		return self.regular_past
 	
 	def purge_past(self):
+		self._past = None
 		self.regular_past.clear()
 		self.reset_integrator(idc_unhandled=True)
 	
